package demoui.menu;

import android.content.Context;
import android.content.res.ColorStateList;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.text.TextPaint;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.Space;
import android.widget.Switch;
import android.widget.TextView;

import androidx.annotation.ColorInt;
import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.annotation.StringRes;
import demoui.menu.MenuItem.OnMenuItemChange;

import static android.widget.ListPopupWindow.MATCH_PARENT;
import static android.widget.ListPopupWindow.WRAP_CONTENT;

/**
 * ˵
 */
public class DropMenu {
    private final static String TAG="DropMenu";
    /**ʽ*/
    private PopupWindow mPopupWindow=null;
    /**ʽêͼ*/
    private View mAnchor=null;
    /**뷽ʽ*/
    private int mGravity=Gravity.NO_GRAVITY;
    /**ʽêͼƫ*/
    private int xOffset=0,yOffset=0;
    /**װز˵*/
    private LinearLayout mContainerLayout =null;
    private Context mContext;
    /**˵*/
    private float menuWidth;
    /**˵߶*/
    private float menuHeight;
    /**˵Ŀƫ*/
    private  float offset;
    private float iconSize;
    private float spaceWidth;
    private float itemHeight;

    public interface OnMenuItemClickListener{
        /**
         * ˵¼
         * @param item Ĳ˵
         * @param view ͼ
         */
        void onMenuItemClick(MenuItem item,View view);
    }
    public interface OnSwitchCheckedChangeListener{
        /**
         * ϰťѡ״̬ʱá
         * @param item ѡťڵĲ˵
         * @param buttonView ״̬Ѹĵĸϰťͼ
         * @param isChecked ťͼѡ״̬
         */
        void onCheckedChanged(MenuItem item,CompoundButton buttonView, boolean isChecked);
    }
    /**ûӲ˵ѡĿʱյ֪ͨ*/
    private OnMenuItemClickListener menuItemClickListener=null;
    /**ûı临ѡť״̬ʱյ֪ͨ*/
    private OnSwitchCheckedChangeListener switchCheckedChangeListener=null;

    /**
     * 캯жλͼµ˵
     * @param context ˵УͨԷʵǰ⡢Դȡ
     * @param anchor ˵ڵêͼпռ䣬ڽʾê·ûпռ䣬ڽʾêϷ
     */
    public DropMenu(@NonNull Context context, @NonNull View anchor){
        this(context,anchor,Gravity.NO_GRAVITY);
    }
    /**
     * 캯жλͼµ˵
     * @param context ˵УͨԷʵǰ⡢Դȡ
     * @param anchor ˵ڵêͼ
     * @param gravity êĶ
     */
    public DropMenu(@NonNull Context context, @NonNull View anchor,int gravity){
      this(context,anchor,0,0,gravity);
    }
    /**
     * 캯жλͼµ˵
     * @param context ˵УͨԷʵǰ⡢Դȡ
     * @param anchor ˵ڵêͼ
     * @param xoff ڶλˮƽƫƣΪλ
     * @param yoff ڶλĴֱƫƣΪλ
     */
    public DropMenu(@NonNull Context context, @NonNull View anchor,int xoff,int yoff){
        this(context,anchor,0,0,Gravity.NO_GRAVITY);
    }
    /**
     * 캯жλͼµ˵
     * @param context ˵УͨԷʵǰ⡢Դȡ
     * @param anchor ˵ڵêͼ
     * @param xoff ڶλˮƽƫƣΪλ
     * @param yoff ڶλĴֱƫƣΪλ
     * @param gravity êĶ
     */
    public DropMenu(@NonNull Context context, @NonNull View anchor,int xoff,int yoff,int gravity){
        this.mContext=context;
        this.mAnchor=anchor;
        this.xOffset=xoff;
        this.yOffset=yoff;
        this.mGravity=gravity;
        this.menuWidth=dp2px(context,100);
        //ԤӰ
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            this.menuHeight = dp2px(context, 5);
        }else{
            this.menuHeight = dp2px(context, 1.5f);
        }
        this.itemHeight=dp2px(context,32);
        this.offset=dp2px(context,5);
        this.iconSize=dp2px(context,24);
        this.spaceWidth=dp2px(context,32);
        createPopup();
    }
    /******************************˵*********************************/
    /**
     * Ӳ˵
     * @param titleResId 
     */
    public boolean addMenuItem(String key,@StringRes int titleResId){
        return addMenuItem(key,titleResId,0);
    }
    /**
     * Ӳ˵
     * @param titleResId 
     * @param resIconId ͼID
     */
    public boolean addMenuItem(String key,@StringRes int titleResId,@DrawableRes int resIconId){
        return addMenuItem(key,titleResId,resIconId,false,false);
    }
    /**
     * Ӳ˵
     * @param titleResId 
     * @param resIconId ͼID
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     */
    public boolean addMenuItem(String key,@StringRes int titleResId,@DrawableRes int resIconId,boolean switchItem,boolean switchItemChecked){
        return addMenuItem(key,titleResId,resIconId,switchItem,switchItemChecked,false);
    }
    /**
     * Ӳ˵
     * @param titleResId 
     * @param resIconId ͼID
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     * @param selectedItem Ŀѡ״̬
     */
    public boolean addMenuItem(String key,@StringRes int titleResId, @DrawableRes int resIconId, boolean switchItem, boolean switchItemChecked, boolean selectedItem){
        return addMenuItem(key,mContext.getResources().getString(titleResId),mContext.getResources().getDrawable(resIconId),switchItem,switchItemChecked,selectedItem);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param resIconId ͼID
     */
    public boolean addMenuItem(String key,String title,@DrawableRes int resIconId){
        return addMenuItem(key,title,resIconId,false,false);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param resIconId ͼID
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     */
    public boolean addMenuItem(String key,String title,@DrawableRes int resIconId,boolean switchItem,boolean switchItemChecked){
        return addMenuItem(key,title,resIconId,switchItem,switchItemChecked,false);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param resIconId ͼID
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     * @param selectedItem Ŀѡ״̬
     */
    public boolean addMenuItem(String key,String title,@DrawableRes int resIconId,boolean switchItem,boolean switchItemChecked,boolean selectedItem){
        return addMenuItem(key,title,mContext.getResources().getDrawable(resIconId),switchItem,switchItemChecked,selectedItem);
    }
    /**
     * Ӳ˵
     * @param title 
     */
    public boolean addMenuItem(String key,String title){
        return addMenuItem(key,title,null);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param icon ͼ
     */
    public boolean addMenuItem(String key,String title,Drawable icon){
        return addMenuItem(key,title,icon,false,false);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param icon ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     */
    public boolean addMenuItem(String key,String title,Drawable icon,boolean switchItem,boolean switchItemChecked){
        return addMenuItem(key,title,icon,switchItem,switchItemChecked,false);
    }
    /**
     * Ӳ˵
     * @param title 
     * @param icon ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     * @param selectedItem Ŀѡ״̬
     */
    public boolean addMenuItem(String key,String title,Drawable icon,boolean switchItem,
                               boolean switchItemChecked,boolean selectedItem){
        return addMenuItem(-1,key,title,icon,switchItem,switchItemChecked,selectedItem);
    }

    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param title 
     * @param resIconId ͼ
     */
    public boolean addMenuItem(int index,String key,String title,@DrawableRes int resIconId){
        return  addMenuItem(index,key,title,resIconId,false,false);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param title 
     * @param resIconId ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     */
    public boolean addMenuItem(int index,String key,String title,@DrawableRes int resIconId,boolean switchItem,boolean switchItemChecked){
        return addMenuItem(index,key,title,resIconId,switchItem,switchItemChecked,false);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param title 
     * @param resIconId ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     * @param selectedItem Ŀѡ״̬
     */
    public boolean addMenuItem(int index,String key,String title,@DrawableRes int resIconId,
                               boolean switchItem,boolean switchItemChecked,boolean selectedItem){
        return addMenuItem(index,key,title,mContext.getResources().getDrawable(resIconId),
                switchItem,switchItemChecked,selectedItem);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param title 
     */
    public boolean addMenuItem(int index,String key,String title){
        return addMenuItem(index,key,title,null);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param title 
     * @param icon ͼ
     */
    public boolean addMenuItem(int index,String key,String title,Drawable icon){
        return addMenuItem(index,key,title,icon,false,false);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param key ˵ؼ
     * @param title 
     * @param icon ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     */
    public boolean addMenuItem(int index,String key,String title,Drawable icon,boolean switchItem,boolean switchItemChecked){
        return addMenuItem(index,key,title,icon,switchItem,switchItemChecked,false);
    }
    /**
     * Ӳ˵ָλ
     * @param index λ(&lt;0 || &gt;=size()ʾӵбĩβ
     * @param key ˵ؼ
     * @param title 
     * @param icon ͼ
     * @param switchItem Switch
     * @param switchItemChecked switchItemΪnullòЧ
     * @param selectedItem Ŀѡ״̬
     */
    public boolean addMenuItem(int index,String key,String title,Drawable icon,boolean switchItem,
                               boolean switchItemChecked,boolean selectedItem){
        MenuItem menuItem=new MenuItem(key,title,icon,switchItem,selectedItem);
        menuItem.setSwitchChecked(switchItemChecked);
        return addMenuItem(index,menuItem);
    }
    /**
     * Ӳ˵˵β
     * @param menuItem
     */
    public boolean addMenuItem(MenuItem menuItem){
        return addMenuItem(-1,menuItem);
    }
    /**
     * Ӳ˵ָλ
     * @param index
     * @param menuItem
     */
    public boolean addMenuItem(int index,MenuItem menuItem){
        if(isMenuItemKeyOnly(menuItem.getItemKey())) {
            if(menuItem.isSelected()){
                for(int pos=0;pos<mContainerLayout.getChildCount();pos++){
                    MenuItem item= (MenuItem)mContainerLayout.getChildAt(pos).getTag();
                    if(item!=null){
                        item.setSelected(false);
                    }
                }
            }
            createMenuItemView(index, menuItem);
            menuItem.setOnMenuItemChange(menuItemChange);  //ò˵ı¼
            return true;
        }else{
            return false;
        }
    }
    /**Ӳ˵ָ*/
    public boolean addSplitLine(){
        return addSplitLine(-1);
    }
    /**
     * Ӳ˵ָߵָλ
     * @param index λ
     * @return
     */
    public boolean addSplitLine(int index){
        String key="splitLine";
        int id=0;
        //Ψһؼ
        do{
            id++;
        }while (!isMenuItemKeyOnly(key+id));
        return addSplitLine(key+id,index);
    }
    /**
     * Ӳ˵ָ
     * @param key ؼ
     * @return
     */
    public boolean addSplitLine(String key){
        return addSplitLine(key,-1);
    }
    /**
     * Ӳ˵ָߵָλ
     * @param key ؼ
     * @param index λ
     * @return
     */
    public boolean addSplitLine(String key,int index){
        if(isMenuItemKeyOnly(key)) {    //ؼǷΨһ
            MenuItem menuItem = new MenuItem(key, true);
            menuItem.setVisibility(true);
            menuItem.setEnabled(false);
            createMenuItemView(index, menuItem);
            return true;
        }else{
            return false;
        }
    }
    /**
     * ɾָ˵
     * @param index ҪϴĲ˵
     * @return
     */
    public boolean  removeMenuItem(int index){
        if(index>=0 && index<mContainerLayout.getChildCount()){
            MenuItem item=(MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(item!=null) {
                //Ŀɾ˵߶
                if (item.isSplitLine()) {
                    menuHeight -= 2 + offset * 2;
                } else {
                    menuHeight -= itemHeight + offset * 2;
                }
                mContainerLayout.removeViewAt(index);
                return true;
            }
        }
        return false;
    }
    /**
     * ɾָ˵
     * @param menuItem
     * @return
     */
    public boolean  removeMenuItem(MenuItem menuItem){
        if(menuItem!=null){
            for(int i=0;i<mContainerLayout.getChildCount();i++){
                if(menuItem.equals(mContainerLayout.getChildAt(i).getTag())){
                    return removeMenuItem(i);
                }
            }
        }
        return false;
    }
        /**
     * ò˵Ŀ¼
     * @param listener
     */
    public void setOnMenuItemClickListener(OnMenuItemClickListener listener) {
        this.menuItemClickListener = listener;
    }

    /**
     * лťѡ״̬ı¼
     * @param listener
     */
    public void setOnSwitchCheckedChangeListener(OnSwitchCheckedChangeListener listener) {
        this.switchCheckedChangeListener = listener;
    }
    /**
     * ػʾָ˵
     * @param index  Ҫִʾ/صĲ˵λ
     * @param visibility ʾ
     */
    public boolean setMenuItemVisibility(int index,boolean visibility){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem = (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (menuItem.isVisibility() != visibility) {
                    menuItem.setVisibility(visibility);
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * ò˵
     * @param index
     * @param enabled
     * @return
     */
    public boolean setMenuItemEnabled(int index,boolean enabled){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem = (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (!menuItem.isSplitLine()) {
                    if (menuItem.isEnabled() != enabled) {
                        menuItem.setEnabled(enabled);
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * ָ˵ѡ״̬
     * @param index
     * @param selected
     * @return
     */
    public boolean setMenuItemSelected(int index,boolean selected){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem =  (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (!menuItem.isSplitLine()) {
                    if (menuItem.isSelected() != selected) {
                        menuItem.setSelected(selected);
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * ò˵ͼ
     * @param index
     * @param resId
     * @return
     */
    public boolean setMenuItemIcon(int index,@DrawableRes int resId){
        return setMenuItemIcon(index,mContext.getResources().getDrawable(resId));
    }
    /**
     * ò˵ͼ
     * @param index
     * @param icon
     * @return
     */
    public boolean setMenuItemIcon(int index,Drawable icon){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem = (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (!menuItem.isSplitLine()) {
                    if (!menuItem.getItemIcon().equals(icon)) {
                        menuItem.setItemIcon(icon);
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * ò˵
     * @param index
     * @param title
     * @return
     */
    public boolean setMenuItemTitle(int index,String title){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem = (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (!menuItem.isSplitLine()) {
                    if (!menuItem.getItemTitle().equals(title)) {
                        menuItem.setItemTitle(title);
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * ò˵ѡ
     * @param index
     * @param _switch
     * @return
     */
    public boolean setMenuItemSwitch(int index,boolean _switch){
        if(index>=0 && index<mContainerLayout.getChildCount()) {
            MenuItem menuItem = (MenuItem)mContainerLayout.getChildAt(index).getTag();
            if(menuItem!=null) {
                if (!menuItem.isSplitLine()) {
                    if (menuItem.isSwitchItem() != _switch) {
                        menuItem.setSwitchItem(_switch);
                        return true;
                    }
                }
            }
        }
        return false;
    }
    /**
     * ò˵Դ
     * @param resid ԴID
     */
    public void setMenuItemBackgroundResource(@DrawableRes int resid){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            mContainerLayout.getChildAt(index).setBackgroundResource(resid);
        }
    }

    /**
     * ָ˵Դ
     * @param resid ԴID
     * @param index ˵
     */
    public void setMenuItemBackgroundResource(@DrawableRes int resid,int index){
        mContainerLayout.getChildAt(index).setBackgroundResource(resid);
    }
    /**
     * ò˵ͼ
     * @param background ͼDrawable
     */
    public void setMenuItemBackgroundDrawable(Drawable background){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                mContainerLayout.getChildAt(index).setBackground(background);
            }else{
                mContainerLayout.getChildAt(index).setBackgroundDrawable(background);
            }
        }
    }
    /**
     * ò˵ͼ
     * @param background ͼDrawable
     * @param index ˵
     */
    public void setMenuItemBackgroundDrawable(Drawable background,int index){
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
            mContainerLayout.getChildAt(index).setBackground(background);
        }else{
            mContainerLayout.getChildAt(index).setBackgroundDrawable(background);
        }
    }
    /**
     * ò˵Ŀɫ
     * @param color
     */
    public void setMenuItemBackgroundColor(@ColorInt int color){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            mContainerLayout.getChildAt(index).setBackgroundColor(color);
        }
    }
    /**
     * ò˵Ŀɫ
     * @param color
     * @param index ˵
     */
    public void setMenuItemBackgroundColor(@ColorInt int color,int index){
        mContainerLayout.getChildAt(index).setBackgroundColor(color);
    }
    /**
     * ÷ָ߱Դid
     * @param resid
     */
    public void setSplitLineResource(@DrawableRes int resid){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
            MenuItem item=(MenuItem)menuItemView.getTag();
            if(item!=null) {
                if (item.isSplitLine() && menuItemView.getChildCount()>0) {
                    menuItemView.getChildAt(0).setBackgroundResource(resid);
                }
            }
        }
    }
    /**
     * ÷ָ߱ͼ
     * @param drawable
     */
    public void setSplitLineDrawable(Drawable drawable){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
            MenuItem item=(MenuItem)menuItemView.getTag();
            if(item!=null) {
                if (item.isSplitLine() && menuItemView.getChildCount()>0) {
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                        menuItemView.getChildAt(0).setBackground(drawable);
                    }else{
                        menuItemView.getChildAt(0).setBackgroundDrawable(drawable);
                    }
                }
            }
        }
    }
    /**
     * ÷ָɫ
     * @param color
     */
    public void setSplitLineColor(@ColorInt int color){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
            MenuItem item=(MenuItem)menuItemView.getTag();
            if(item!=null) {
                if (item.isSplitLine() && menuItemView.getChildCount()>0) {
                    menuItemView.getChildAt(0).setBackgroundColor(color);
                }
            }
        }
    }

    /**
     * ò˵ɫ
     * @param color
     */
    public void setMenuItemTextColor(@ColorInt int color){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
            MenuItem item=(MenuItem)menuItemView.getTag();
            if(item!=null) {
                if (!item.isSplitLine() && menuItemView.getChildCount()>0) {
                    for(int i=0;i<menuItemView.getChildCount();i++){
                        if(menuItemView.getChildAt(i)instanceof TextView){
                            ((TextView) menuItemView.getChildAt(i)).setTextColor(color);
                        }
                    }
                }
            }
        }
    }
    /**
     * ò˵ɫ
     * @param colors
     */
    public void setMenuItemTextColor(ColorStateList colors){
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
            MenuItem item=(MenuItem)menuItemView.getTag();
            if(item!=null) {
                if (!item.isSplitLine() && menuItemView.getChildCount()>0) {
                    for(int i=0;i<menuItemView.getChildCount();i++){
                        if(menuItemView.getChildAt(i)instanceof TextView){
                            ((TextView) menuItemView.getChildAt(i)).setTextColor(colors);
                        }
                    }
                }
            }
        }
    }
        /**
     * ò˵ͼɫ
     * @param color
     */
   public void setMenuItemIconTint(@ColorInt int color){
//       for(int index=0;index<mContainerLayout.getChildCount();index++){
//           LinearLayout menuItemView= (LinearLayout) mContainerLayout.getChildAt(index);
//           MenuItem item=(MenuItem)menuItemView.getTag();
//           if(item!=null) {
//               if (!item.isSplitLine() && menuItemView.getChildCount()>0) {
//                   for(int i=0;i<menuItemView.getChildCount();i++){
//                       if(menuItemView.getChildAt(i)instanceof ImageView){
//                           ((ImageView) menuItemView.getChildAt(i)).setColorFilter(color, PorterDuff.Mode.SRC_IN);
//                       }
//                   }
//               }
//           }
//       }
   }
    /**ȡ˵ȣ˵ȡ*/
    public float getMenuWidth(){
        return menuWidth;
    }
    /**ȡ˵߶ȣ˵ȡ*/
    public float getMenuHeight() {
        return menuHeight;
    }
    /**ȡ߶*/
    public float getItemHeight() {
        return itemHeight;
    }
    /**ʾ˵*/
    public void show(){
        if(mPopupWindow!=null) {
            mPopupWindow.setHeight((int) menuHeight);
            mPopupWindow.setWidth((int) menuWidth);
            if(xOffset==0 && yOffset==0 && mGravity==Gravity.NO_GRAVITY){
                mPopupWindow.showAsDropDown(mAnchor);
            }else{
                int xoff,yoff;
                if(mGravity==Gravity.LEFT||mGravity==Gravity.START
                        ||mGravity==(Gravity.LEFT|Gravity.BOTTOM)
                        ||mGravity==(Gravity.START|Gravity.BOTTOM)){    //ê
                    mPopupWindow.showAsDropDown(mAnchor,xOffset,yOffset);
                }else if(mGravity==Gravity.RIGHT||mGravity==Gravity.END
                        ||mGravity==(Gravity.RIGHT|Gravity.BOTTOM)
                        ||mGravity==(Gravity.END|Gravity.BOTTOM)) { //ê
                    xoff=(xOffset+-(int)(menuWidth-mAnchor.getWidth()));
                    mPopupWindow.showAsDropDown(mAnchor, xoff, yOffset);
                }else if(mGravity==(Gravity.LEFT|Gravity.TOP)
                        ||mGravity==(Gravity.START|Gravity.TOP)){   //
                    yoff=yOffset+-(int)( menuHeight+mAnchor.getHeight());
                    mPopupWindow.showAsDropDown(mAnchor, xOffset, yoff);
                }else if(mGravity==(Gravity.RIGHT|Gravity.TOP)
                        ||mGravity==(Gravity.END|Gravity.TOP)){  //
                     xoff=(xOffset+-(int)(menuWidth-mAnchor.getWidth()));
                     yoff=yOffset+-(int)( menuHeight+mAnchor.getHeight());
                    mPopupWindow.showAsDropDown(mAnchor, xoff,yoff);
                }else{
                    mPopupWindow.showAsDropDown(mAnchor,xOffset,yOffset);
                }
            }
        }
    }
    /**رյ˵*/
    public void dismiss(){
        if(mPopupWindow!=null)
            mPopupWindow.dismiss();
    }
    /***************************ڲ*******************************/
    /***/
    private void createPopup(){
        mPopupWindow=new PopupWindow();
        ColorDrawable dw = new ColorDrawable(Color.parseColor("#70aa00bc"));
        mPopupWindow.setBackgroundDrawable(dw);
        mPopupWindow.setOutsideTouchable(true);
        mPopupWindow.setFocusable(true);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mPopupWindow.setElevation(0f);
        }
        mPopupWindow.setTouchable(true);
        mContainerLayout =new LinearLayout(mContext);
        LinearLayout.LayoutParams params=new LinearLayout.LayoutParams(MATCH_PARENT,WRAP_CONTENT);
        mContainerLayout.setLayoutParams(params);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mContainerLayout.setElevation(dp2px(mContext,5));
        }else{
            mContainerLayout.setTranslationX(dp2px(mContext,-0.5f));
            mContainerLayout.setTranslationY(dp2px(mContext,-1.5f));
        }
        mContainerLayout.setBackgroundColor(Color.WHITE);
        mContainerLayout.setOrientation(LinearLayout.VERTICAL);
        mContainerLayout.setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
        mContainerLayout.setOnSystemUiVisibilityChangeListener(new View.OnSystemUiVisibilityChangeListener() {
            @Override
            public void onSystemUiVisibilityChange(int visibility) {
//                int uiOptions = View.SYSTEM_UI_FLAG_LAYOUT_STABLE |
//                        //λ״̬?
//                        View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION |
//                        //ȫ
//                        View.SYSTEM_UI_FLAG_FULLSCREEN |
//                        //ص
//                        View.SYSTEM_UI_FLAG_HIDE_NAVIGATION |
//                        View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN;
                               
                int uiOptions = View.SYSTEM_UI_FLAG_LAYOUT_STABLE |
                        //λ״̬?
                        View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION;
                
                uiOptions |= 0x00001000;
                mContainerLayout.setSystemUiVisibility(uiOptions);
            }
        });
        mPopupWindow.setContentView(mContainerLayout);
    }

    /**
     * ˵
     * @param index λ
     * @param menuItem ˵Ŀ
     */
    private void createMenuItemView(int index,MenuItem menuItem){
        /*****************˵ͼ********************/
        LinearLayout menuItemView=new LinearLayout(mContext);
        menuItemView.setOrientation(LinearLayout.HORIZONTAL);
        if(menuItem.isSplitLine()){
            View view=new View(mContext);
            view.setBackgroundColor(0x60888888);   //ĬϷָɫ
            LinearLayout.LayoutParams layoutParams=new LinearLayout.LayoutParams(MATCH_PARENT, 2, 0);
            layoutParams.setMargins((int)offset*2,(int)offset,(int)offset*2,(int)offset);
            view.setLayoutParams(layoutParams);
            menuItemView.setTag(menuItem);  //˵Ŀݰtag
            menuItemView.addView(view);
            if(menuItem.isVisibility()) {
                menuItemView.setVisibility(View.VISIBLE);
                menuHeight+=2+offset*2;
            }else{
                menuItemView.setVisibility(View.GONE);
            }
            if(index>=0){
                mContainerLayout.addView(menuItemView,index);
            }else {
                mContainerLayout.addView(menuItemView);
            }
        }else {
            menuItemView.setClickable(true);
            menuItemView.setFocusable(true);
            menuItemView.setBackgroundResource(android.R.drawable.menuitem_background);
            menuItemView.setLayoutParams(new LinearLayout.LayoutParams(MATCH_PARENT, WRAP_CONTENT, 0));
            menuItemView.setPadding(0, (int)offset, 0, (int)offset);
            menuItemView.setGravity(Gravity.CENTER_VERTICAL); //ͼֱж
            /****************ò˵ͼ*****************/
            ImageView iconView=null;
            LinearLayout.LayoutParams iconParams=null;
            if(menuItem.getItemIcon()!=null) {
                iconView = new ImageView(mContext); //˵ͼ
                iconView.setImageDrawable(menuItem.getItemIcon());
                iconParams = new LinearLayout.LayoutParams((int) iconSize, (int) iconSize, 0);
                iconParams.leftMargin = (int) offset;
                iconParams.rightMargin = (int) offset / 2;
                iconView.setLayoutParams(iconParams);
            }
            /*******************ò˵ǩ***********************/
            TextView titleView = new TextView(mContext);  //˵ǩ
            titleView.setText(menuItem.getItemTitle());
            titleView.setGravity(Gravity.CENTER_VERTICAL);  //ǩֱж
            titleView.setSingleLine(true);    //ı
            titleView.setEllipsize(TextUtils.TruncateAt.END);  //βʡ
            LinearLayout.LayoutParams titleParams = new LinearLayout.LayoutParams(MATCH_PARENT, (int) itemHeight, 1);
            titleParams.leftMargin = (int)(iconView!=null?(offset / 2):offset);
            titleParams.rightMargin = (int)offset;
            titleView.setLayoutParams(titleParams);
            TextPaint textPaint = titleView.getPaint();
            float textWidth = textPaint.measureText(menuItem.getItemTitle()); //õı
            //Ϊıм3dpｫı߶ȱ3dp
            //float textHeight=textPaint.descent()-textPaint.ascent()+dp2px(mContext,3);
            Log.e(TAG, ":" + textWidth);
            /****************ռλͼ*******************/
            Space space = new Space(mContext);
            space.setLayoutParams(new LinearLayout.LayoutParams((int)this.spaceWidth, 0));
            /****************ͼͱǩӵͼ***************/
            if(iconView!=null)
                menuItemView.addView(iconView);
            menuItemView.addView(titleView);
            menuItemView.addView(space);
            /******************switchItem****************/
            int switchWidth=0;
            Switch mSwidth=null;
            if(menuItem.isSwitchItem()){
                mSwidth=new Switch(mContext);
                int w=(int)dp2px(mContext,50);
                LinearLayout.LayoutParams switchParams = new LinearLayout.LayoutParams(w, (int) itemHeight, 0);
                switchParams.rightMargin= (int) offset;
                mSwidth.setLayoutParams(switchParams);
                mSwidth.setChecked(menuItem.isSwitchChecked());   //ѡ״̬
                switchWidth=switchParams.rightMargin+w;
                menuItemView.addView(mSwidth);
                // mSwidth.setChecked();
            }
            menuItemView.setSelected(menuItem.isSelected());  //ѡ״̬
            /*********˵ӵ˵ͼ***********/
            menuItemView.setTag(menuItem);  //˵Ŀݰtag
            menuItemView.setEnabled(menuItem.isEnabled());
            if(menuItem.isVisibility()){
                menuItemView.setVisibility(View.VISIBLE);
                //ǰ=ı+ıͼұλ+ͼ+ͼұλ+ռλͼ+лؿ
                float tempWidth = textWidth +titleParams.leftMargin + titleParams.rightMargin +
                        (iconParams!=null?iconSize + iconParams.leftMargin + iconParams.rightMargin:0) +
                        spaceWidth+switchWidth;
                if (tempWidth > menuWidth) {
                    menuWidth = (int) tempWidth;
                }
                menuHeight+=itemHeight+offset*2;   //̶Ŀ߶
            }else{
                menuItemView.setVisibility(View.GONE);
            }
            if(index<0 ||index>=mContainerLayout.getChildCount()){
                mContainerLayout.addView(menuItemView);
            }else {
                mContainerLayout.addView(menuItemView,index);
            }
            if(mSwidth!=null)
                mSwidth.setOnCheckedChangeListener(switchChangeListener);   //ı
            menuItemView.setOnClickListener(clickListener);    //ò˵ͼ¼
        }
    }
    /**
     * ָؼǷΪ˵бΨһؼ
     * @param key ҪĹؼ
     * @return true Ψһfalse ظ
     */
    private boolean isMenuItemKeyOnly(String key){
        for(int pos=0;pos<mContainerLayout.getChildCount();pos++){
            MenuItem item=(MenuItem) mContainerLayout.getChildAt(pos).getTag();
            if(item!=null){
                if(item.getItemKey().equals(key)){
                    return false;
                }
            }
        }
        return true;
    }
    /**
     * ˵ҪĿ
     * @param currentMenuItemView
     * @return
     */
    private float getMenuItemWidth(LinearLayout currentMenuItemView){
        float textWidth=0,imgWidth=0,switchWidth=0;
        for(int i=0;i<currentMenuItemView.getChildCount();i++){
            View view=currentMenuItemView.getChildAt(i);
            LinearLayout.LayoutParams params= (LinearLayout.LayoutParams) view.getLayoutParams();
            if(view instanceof TextView) {
                TextPaint textPaint = ((TextView) view).getPaint();
                textWidth = textPaint.measureText(((TextView) view).getText().toString()); //õı
                textWidth += params.leftMargin + params.rightMargin;
            }else if(view instanceof ImageView){
                imgWidth=iconSize+params.leftMargin+params.rightMargin;
            }else if(view instanceof Switch){
                float w= dp2px(mContext,50);
                switchWidth=w+params.rightMargin;
            }
        }
        return textWidth+imgWidth+this.spaceWidth+switchWidth;
    }
    /**¼㲢ò˵*/
    private void setMenuWidth(){
        float mWidth=0;
        for(int index=0;index<mContainerLayout.getChildCount();index++){
            float tempWidth=getMenuItemWidth((LinearLayout) mContainerLayout.getChildAt(index));
            if(tempWidth>mWidth)
                mWidth=tempWidth;
        }
        menuWidth= mWidth;
    }
    private float sp2px(Context context, float spValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return spValue * fontScale ;
    }
    private float dp2px(Context context, float dpValue) {
        final float fontScale = context.getResources().getDisplayMetrics().density;
        return dpValue * fontScale ;
    }
    /**˵ı¼*/
    private MenuItem.OnMenuItemChange menuItemChange=new MenuItem.OnMenuItemChange() {
        @Override
        public void onChange(MenuItem item, String attributeName, Object value) {
            for(int id=0;id<mContainerLayout.getChildCount();id++) {
                LinearLayout menuItemView=(LinearLayout) mContainerLayout.getChildAt(id);
                MenuItem menuItem=(MenuItem)menuItemView.getTag();
                if(item.equals(menuItem)){
                    //Log.e(TAG,attributeName+"ı");
                    if(attributeName.equals("selected")) {
                        menuItemView.setSelected((Boolean) value);
                    }else if(attributeName.equals("enabled")){
                        boolean enabled=(Boolean)value;
                        menuItemView.setEnabled(enabled);
                        if(!item.isSplitLine() && item.isSwitchItem()){
                            if(menuItemView.getChildAt(menuItemView.getChildCount()-1)instanceof Switch){
                                menuItemView.getChildAt(menuItemView.getChildCount()-1).setEnabled(enabled);
                            }
                        }
                    }else if(attributeName.equals("visibility")){
                        boolean visibility=(Boolean) value;
                        if(visibility){
                            if(menuItemView.getVisibility()==View.GONE){
                                if(item.isSplitLine()){
                                    menuHeight-=2+offset*2;
                                }else{
                                    menuHeight-=itemHeight+offset*2;
                                }
                                menuItemView.setVisibility(View.VISIBLE);
                            }
                        }else{
                            if(menuItemView.getVisibility()!=View.GONE){
                                if(item.isSplitLine()){
                                    menuHeight+=2+offset*2;
                                }else{
                                    menuHeight+=itemHeight+offset*2;
                                }
                                menuItemView.setVisibility(View.GONE);
                            }
                        }
                    }else if(attributeName.equals("switchItem")){
                        boolean val=(Boolean)value;
                        if(val){
                            if(!(menuItemView.getChildAt(menuItemView.getChildCount()-1) instanceof Switch)){
                                Switch mSwidth=new Switch(mContext);
                                mSwidth.setChecked(item.isSwitchChecked());
                                menuItemView.addView(mSwidth);
                                mSwidth.setOnCheckedChangeListener(switchChangeListener);  //
                            }
                            //Ŀ¿ؼʱֱӱȶԵõȼ
                            float itemWidth=getMenuItemWidth(menuItemView);
                            if(itemWidth>menuWidth)
                                menuWidth=itemWidth;
                        }else{
                            if((menuItemView.getChildAt(menuItemView.getChildCount()-1) instanceof Switch)){
                                menuItemView.removeViewAt(menuItemView.getChildCount()-1);
                                setMenuWidth();    //ǼͼҪ¼в˵õ¿
                            }
                        }
                    }else if(attributeName.equals("itemIcon")){
                        Drawable icon= (Drawable) value;
                        if(icon==null){
                            if(menuItemView.getChildAt(0)instanceof ImageView){
                                menuItemView.removeViewAt(0);
                                setMenuWidth();    //ǼͼҪ¼в˵õ¿
                            }
                        }else{
                            ImageView iconView;
                            if(menuItemView.getChildAt(0)instanceof ImageView){
                                iconView= (ImageView) menuItemView.getChildAt(0);
                                if(!iconView.getDrawable().equals(icon)){
                                    iconView.setImageDrawable(icon);
                                }
                            }else{
                                iconView=new ImageView(mContext);
                                iconView.setImageDrawable(menuItem.getItemIcon());
                                LinearLayout.LayoutParams iconParams = new LinearLayout.LayoutParams((int) iconSize, (int) iconSize, 0);
                                iconParams.leftMargin = (int) offset;
                                iconParams.rightMargin = (int) offset / 2;
                                iconView.setLayoutParams(iconParams);
                                menuItemView.addView(iconView,0);
                                //Ŀ¿ؼʱֱӱȶԵõȼ
                                float itemWidth=getMenuItemWidth(menuItemView);
                                if(itemWidth>menuWidth)
                                    menuWidth=itemWidth;
                            }
                        }
                    }else if(attributeName.equals("itemTitle")){
                        String title= (String) value;
                        if(title==null)
                            title="";
                        for(int i=0;i<menuItemView.getChildCount();i++){
                            if(menuItemView.getChildAt(i)instanceof TextView){
                                TextView textView= (TextView) menuItemView.getChildAt(i);
                                if(!textView.getText().equals(title)){
                                    textView.setText(title);
                                    setMenuWidth();    //ڱǩݳȲȷֱ¼˵
                                    return;
                                }
                            }
                        }
                    }
                    return;
                }
            }
        }
    };
    /**
     * Switch״̬ı¼ӿڶ
     */
    private OnCheckedChangeListener switchChangeListener=new OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            if(switchCheckedChangeListener!=null){
                LinearLayout menuItemView= (LinearLayout) buttonView.getParent();
                if(menuItemView!=null){
                    MenuItem item=(MenuItem)menuItemView.getTag();
                    switchCheckedChangeListener.onCheckedChanged(item,buttonView,isChecked);
                }
            }
            Log.e(TAG,((MenuItem) ((LinearLayout)buttonView.getParent()).getTag()).getItemKey()+"˵ѡťı");
        }
    };
    private View.OnClickListener clickListener=new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            mPopupWindow.dismiss();
            if(menuItemClickListener!=null){
                menuItemClickListener.onMenuItemClick((MenuItem) v.getTag(),v);
            }
            Log.e(TAG,""+((MenuItem) v.getTag()).getItemKey()+"˵");
        }
    };
}



